@@ -14,6 +14,9 @@
 //
 // SPDX-License-Identifier: Apache-2.0
 
+/*
+OPKSSH is a command-line tool that allows users to authenticate with OpenID Connect providers and generate SSH keys for secure access to servers.
+*/
 package main
 
 import (
@@ -28,10 +31,16 @@ import (
 	"strings"
 	"syscall"
 
+	"github.com/thediveo/enumflag/v2"
+
 	"github.com/openpubkey/opkssh/commands"
+	config "github.com/openpubkey/opkssh/commands/config"
 	"github.com/openpubkey/opkssh/policy"
 	"github.com/openpubkey/opkssh/policy/files"
+	"github.com/spf13/afero"
 	"github.com/spf13/cobra"
+	"golang.org/x/term"
+	"text/tabwriter"
 )
 
 var (
@@ -125,6 +134,7 @@ Arguments:
 	var disableBrowserOpenArg bool
 	var printIdTokenArg bool
 	var keyPathArg string
+	var keyTypeArg commands.KeyType
 	loginCmd := &cobra.Command{
 		SilenceUsage: true,
 		Use:          "login [alias]",
@@ -155,7 +165,7 @@ Arguments:
 				providerAliasArg = args[0]
 			}
 
-			login := commands.NewLogin(autoRefreshArg, configPathArg, createConfigArg, configureArg, logDirArg, sendAccessTokenArg, disableBrowserOpenArg, printIdTokenArg, providerArg, keyPathArg, providerAliasArg)
+			login := commands.NewLogin(autoRefreshArg, configPathArg, createConfigArg, configureArg, logDirArg, sendAccessTokenArg, disableBrowserOpenArg, printIdTokenArg, providerArg, keyPathArg, providerAliasArg, keyTypeArg)
 			if err := login.Run(ctx); err != nil {
 				log.Println("Error executing login command:", err)
 				return err
@@ -176,6 +186,7 @@ Arguments:
 	loginCmd.Flags().BoolVar(&sendAccessTokenArg, "send-access-token", false, "Set this flag to send the Access Token as well as the PK Token in the SSH cert. The Access Token is used to call the userinfo endpoint to get claims not included in the ID Token")
 	loginCmd.Flags().StringVar(&providerArg, "provider", "", "OpenID Provider specification in the format: <issuer>,<client_id> or <issuer>,<client_id>,<client_secret> or <issuer>,<client_id>,<client_secret>,<scopes>")
 	loginCmd.Flags().StringVarP(&keyPathArg, "private-key-file", "i", "", "Path where private keys is written")
+	loginCmd.Flags().VarP(enumflag.New(&keyTypeArg, "Key Type", map[commands.KeyType][]string{commands.ECDSA: {commands.ECDSA.String()}, commands.ED25519: {commands.ED25519.String()}}, enumflag.EnumCaseInsensitive), "key-type", "t", "Type of key to generate")
 	rootCmd.AddCommand(loginCmd)
 
 	readhomeCmd := &cobra.Command{
@@ -275,7 +286,7 @@ Arguments:
 			}
 
 			v := commands.NewVerifyCmd(*pktVerifier, commands.OpkPolicyEnforcerFunc(userArg), serverConfigPathArg)
-			if err := v.SetEnvVarInConfig(); err != nil {
+			if err := v.ReadFromServerConfig(); err != nil {
 				log.Println("Failed to set environment variables in config:", err)
 			}
 
@@ -293,6 +304,69 @@ Arguments:
 	verifyCmd.Flags().StringVar(&serverConfigPathArg, "config-path", "/etc/opk/config.yml", "Path to the server config file. Default: /etc/opk/config.yml.")
 	rootCmd.AddCommand(verifyCmd)
 
+	clientCmd := &cobra.Command{
+		Use:     "client [subcommand]",
+		Short:   "Interact with client configuration",
+		Example: `  opkssh client provider list`,
+		Args:    cobra.ExactArgs(0),
+	}
+
+	providerCmd := &cobra.Command{
+		Use:     "provider [subcommand]",
+		Short:   "Interact with provider configuration",
+		Example: `  opkssh client provider list`,
+		Args:    cobra.ExactArgs(0),
+	}
+
+	providerListCmd := &cobra.Command{
+		Use:     "list",
+		Short:   "List configured providers",
+		Example: `  opkssh client provider list`,
+		Args:    cobra.ExactArgs(0),
+		RunE: func(cmd *cobra.Command, args []string) error {
+			client_config, err := config.GetClientConfigFromFile(configPathArg, afero.NewOsFs())
+
+			if err != nil {
+				log.Fatal("Unable to load providers. ", err)
+			}
+
+			isTTY := term.IsTerminal(int(os.Stdout.Fd()))
+
+			var w *tabwriter.Writer
+			if isTTY {
+				// Nice aligned table for TTY output
+				w = tabwriter.NewWriter(os.Stdout, 0, 0, 2, ' ', 0)
+				fmt.Fprintln(w, "Alias\tIssuer")
+				fmt.Fprintln(w, "-----\t------")
+			} else {
+				// Simpler formatting for non-TTY (e.g., when piping to a file)
+				w = tabwriter.NewWriter(os.Stdout, 0, 0, 1, ' ', tabwriter.DiscardEmptyColumns)
+			}
+
+			for _, p := range client_config.Providers {
+				for _, alias := range p.AliasList {
+					fmt.Fprintf(w, "%s\t%s\n", alias, p.Issuer)
+				}
+			}
+			w.Flush()
+
+			// and lets check it can be loaded into a map, after we print the contents
+			if _, err = config.CreateProvidersMap(client_config.Providers); err != nil {
+				log.Fatal("Unable to parse providers. ", err)
+			}
+
+			return nil
+		},
+	}
+
+	providerListCmd.Flags().StringVar(&configPathArg, "config-path", "", "Path to the client config file. Default: ~/.opk/config.yml on linux and %APPDATA%\\.opk\\config.yml on windows.")
+
+	providerCmd.AddCommand(providerListCmd)
+
+	clientCmd.AddCommand(providerCmd)
+
+	rootCmd.AddCommand(clientCmd)
+
 	err := rootCmd.Execute()
 	if err != nil {
 		return 1
